@@ -378,8 +378,7 @@ inline float
 Node::to_float() const
 {
 	assert(is_float());
-	char* endptr;
-	return serd_strtod((const char*)sord_node_get_string(_c_obj), &endptr);
+	return serd_strtod((const char*)sord_node_get_string(_c_obj), NULL);
 }
 
 inline bool
@@ -473,10 +472,8 @@ public:
 	inline World& world() const { return _world; }
 
 private:
-	World&      _world;
-	Node        _base;
-	SerdWriter* _writer;
-	size_t      _next_blank_id;
+	World& _world;
+	Node   _base;
 };
 
 /** Create an empty in-memory RDF model.
@@ -488,7 +485,6 @@ Model::Model(World&             world,
              bool               graphs)
 	: _world(world)
 	, _base(world, Node::URI, base_uri)
-	, _writer(NULL)
 {
 	_c_obj = sord_new(_world.world(), indices, graphs);
 }
@@ -599,6 +595,10 @@ Model::write_to_string(const std::string& base_uri_str,
 	                                     string_sink,
 	                                     &ret);
 
+	const SerdNode base_uri_node = serd_node_from_string(
+		SERD_URI, (const uint8_t*)base_uri_str.c_str());
+	serd_writer_set_base_uri(writer, &base_uri_node);
+
 	serd_env_foreach(_world.prefixes().c_obj(),
 	                 (SerdPrefixSink)serd_writer_set_prefix,
 	                 writer);
@@ -650,4 +650,3 @@ Model::get(const Node& subject,
 }  // namespace Sord
 
 #endif  // SORD_SORDMM_HPP
-
