@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2013 David Robillard <http://drobilla.net>
+  Copyright 2011-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -403,6 +403,13 @@ main(int argc, char** argv)
 	sord_node_free(world, badns);
 	serd_env_free(env);
 
+	// Attempt to create node from garbage
+	SerdNode junk = SERD_NODE_NULL;
+	junk.type = 1234;
+	if (sord_node_from_serd_node(world, env, &junk, NULL, NULL)) {
+		return test_fail("Successfully created node from garbage serd node\n");
+	}
+
 	// Attempt to create NULL node
 	SordNode* nil_node = sord_node_from_serd_node(
 		world, NULL, &SERD_NODE_NULL, NULL, NULL);
@@ -515,19 +522,36 @@ main(int argc, char** argv)
 		goto fail;
 	}
 
-	size_t len;
-	const uint8_t* str = sord_node_get_string_counted(lit_id2, &len);
-	if (strcmp((const char*)str, "hello")) {
-		return test_fail("Literal node corrupt\n");
-	} else if (len != strlen("hello")) {
-		return test_fail("Literal length incorrect\n");
-	}
-
 	if (sord_num_nodes(world) != initial_num_nodes) {
 		return test_fail("Num nodes %zu != %zu\n",
 		                 sord_num_nodes(world), initial_num_nodes);
 	}
 
+	const uint8_t ni_hao[] = { 0xE4, 0xBD, 0xA0, 0xE5, 0xA5, 0xBD };
+	SordNode*     chello   = sord_new_literal(world, NULL, ni_hao, "cmn");
+
+	// Test literal length
+	size_t         n_bytes;
+	size_t         n_chars;
+	const uint8_t* str = sord_node_get_string_counted(lit_id2, &n_bytes);
+	if (strcmp((const char*)str, "hello")) {
+		return test_fail("Literal node corrupt\n");
+	} else if (n_bytes != strlen("hello")) {
+		return test_fail("ASCII literal byte count incorrect\n");
+	}
+
+	str = sord_node_get_string_measured(lit_id2, &n_bytes, &n_chars);
+	if (n_bytes != strlen("hello") || n_chars != strlen("hello")) {
+		return test_fail("ASCII literal measured length incorrect\n");
+	}
+
+	str = sord_node_get_string_measured(chello, &n_bytes, &n_chars);
+	if (n_bytes != 6) {
+		return test_fail("Multi-byte literal byte count incorrect\n");
+	} else if (n_chars != 2) {
+		return test_fail("Multi-byte literal character count incorrect\n");
+	}
+
 	// Check interning doesn't clash non-equivalent values
 	SordNode* uri_id3   = sord_new_uri(world, USTR("http://example.orgX"));
 	SordNode* blank_id3 = sord_new_blank(world, USTR("testblankX"));
@@ -639,6 +663,9 @@ main(int argc, char** argv)
 	}
 	sord_iter_free(iter);
 
+	// Test double remove (silent success)
+	sord_remove(sord, tup);
+
 	// Load a couple graphs
 	SordNode* graph42 = uri(world, 42);
 	SordNode* graph43 = uri(world, 43);
@@ -657,6 +684,13 @@ main(int argc, char** argv)
 	}
 	sord_iter_free(iter);
 
+	// Erase the first tuple (an element in the default graph)
+	iter = sord_begin(sord);
+	if (sord_erase(sord, iter)) {
+		return test_fail("Failed to erase begin iterator on non-empty model\n");
+	}
+	sord_iter_free(iter);
+
 	// Ensure only the other graph is left
 	SordQuad quad;
 	SordQuad pat = { 0, 0, 0, graph42 };
@@ -686,6 +720,8 @@ main(int argc, char** argv)
 		fprintf(stderr, "Failed to re-read string (%s)\n", serd_strerror(st));
 		goto fail;
 	}
+	serd_reader_free(reader);
+	serd_env_free(env);
 
 	// Ensure we only see triple once
 	size_t n_triples = 0;
@@ -698,6 +734,7 @@ main(int argc, char** argv)
 
 		++n_triples;
 	}
+	sord_iter_free(iter);
 	if (n_triples != 1) {
 		fprintf(stderr, "Found duplicate triple\n");
 		goto fail;
@@ -710,6 +747,7 @@ main(int argc, char** argv)
 	for (iter = sord_begin(sord); !sord_iter_end(iter); sord_iter_next(iter)) {
 		++n_triples;
 	}
+	sord_iter_free(iter);
 
 	sord_free(sord);
 	sord_world_free(world);
